@@ -1,124 +1,8 @@
-#' Compute the LOO information criterion
-#' 
-#' Perform approximate leave-one-out cross-validation based 
-#' on the posterior likelihood using the \pkg{loo} package.
-#' 
-#' @aliases LOO.brmsfit loo.brmsfit loo
-#' 
-#' @param x A fitted model object.
-#' @param ... More fitted model objects or further arguments
-#'   passed to the underlying post-processing functions.
-#' @param compare A flag indicating if the information criteria
-#'  of the models should be compared to each other
-#'  via \code{\link{compare_ic}}.
-#' @param pointwise A flag indicating whether to compute the full
-#'  log-likelihood matrix at once or separately for each observation. 
-#'  The latter approach is usually considerably slower but 
-#'  requires much less working memory. Accordingly, if one runs 
-#'  into memory issues, \code{pointwise = TRUE} is the way to go.
-#'  By default, \code{pointwise} is automatically chosen based on 
-#'  the size of the model.
-#' @param reloo Logical; Indicate whether \code{\link{reloo}} 
-#'  should be applied on problematic observations. Defaults to \code{FALSE}.
-#' @param k_threshold The threshold at which pareto \eqn{k} 
-#'   estimates are treated as problematic. Defaults to \code{0.7}. 
-#'   Only used if argument \code{reloo} is \code{TRUE}.
-#'   See \code{\link[loo:pareto_k_ids]{pareto_k_ids}} for more details.
-#' @param model_names If \code{NULL} (the default) will use model names 
-#'   derived from deparsing the call. Otherwise will use the passed 
-#'   values as model names.
-#' @inheritParams predict.brmsfit
-#' 
-#' @details When comparing models fitted to the same data, 
-#'  the smaller the LOO, the better the fit.
-#'  For \code{brmsfit} objects, \code{loo} is an alias of \code{LOO}.
-#'  Use method \code{\link{add_ic}} to store
-#'  information criteria in the fitted model object for later usage.
-#'  
-#' @return If just one object is provided, an object of class \code{ic}. 
-#'  If multiple objects are provided, an object of class \code{iclist}.
-#' 
-#' @author Paul-Christian Buerkner \email{paul.buerkner@@gmail.com}
-#' 
-#' @examples
-#' \dontrun{
-#' # model with population-level effects only
-#' fit1 <- brm(rating ~ treat + period + carry,
-#'             data = inhaler, family = "gaussian")
-#' LOO(fit1)
-#' 
-#' # model with an additional varying intercept for subjects
-#' fit2 <- brm(rating ~ treat + period + carry + (1|subject),
-#'             data = inhaler, family = "gaussian")
-#' # compare both models
-#' LOO(fit1, fit2)                          
-#' }
-#' 
-#' @references 
-#' Vehtari, A., Gelman, A., & Gabry J. (2016). Practical Bayesian model
-#' evaluation using leave-one-out cross-validation and WAIC. In Statistics 
-#' and Computing, doi:10.1007/s11222-016-9696-4. arXiv preprint arXiv:1507.04544.
-#' 
-#' Gelman, A., Hwang, J., & Vehtari, A. (2014). 
-#' Understanding predictive information criteria for Bayesian models. 
-#' Statistics and Computing, 24, 997-1016.
-#' 
-#' Watanabe, S. (2010). Asymptotic equivalence of Bayes cross validation 
-#' and widely applicable information criterion in singular learning theory. 
-#' The Journal of Machine Learning Research, 11, 3571-3594.
-#' 
 #' @export
 LOO <- function(x, ...) {
   UseMethod("LOO")
 }
 
-#' Compute the WAIC
-#' 
-#' Compute the widely applicable information criterion (WAIC)
-#' based on the posterior likelihood using the \pkg{loo} package.
-#' 
-#' @aliases WAIC.brmsfit waic.brmsfit waic
-#' 
-#' @inheritParams LOO
-#' 
-#' @details When comparing models fitted to the same data, 
-#'  the smaller the WAIC, the better the fit.
-#'  For \code{brmsfit} objects, \code{waic} is an alias of \code{WAIC}.
-#'  Use method \code{\link[brms:add_ic]{add_ic}} to store
-#'  information criteria in the fitted model object for later usage.
-#'  
-#' @return If just one object is provided, an object of class \code{ic}. 
-#'  If multiple objects are provided, an object of class \code{iclist}.
-#' 
-#' @author Paul-Christian Buerkner \email{paul.buerkner@@gmail.com}
-#' 
-#' @examples
-#' \dontrun{
-#' # model with population-level effects only
-#' fit1 <- brm(rating ~ treat + period + carry,
-#'             data = inhaler, family = "gaussian")
-#' WAIC(fit1)
-#' 
-#' # model with an additional varying intercept for subjects
-#' fit2 <- brm(rating ~ treat + period + carry + (1|subject),
-#'             data = inhaler, family = "gaussian")
-#' # compare both models
-#' WAIC(fit1, fit2)                          
-#' }
-#' 
-#' @references 
-#' Vehtari, A., Gelman, A., & Gabry J. (2016). Practical Bayesian model
-#' evaluation using leave-one-out cross-validation and WAIC. In Statistics 
-#' and Computing, doi:10.1007/s11222-016-9696-4. arXiv preprint arXiv:1507.04544.
-#' 
-#' Gelman, A., Hwang, J., & Vehtari, A. (2014). 
-#' Understanding predictive information criteria for Bayesian models. 
-#' Statistics and Computing, 24, 997-1016.
-#' 
-#' Watanabe, S. (2010). Asymptotic equivalence of Bayes cross validation 
-#' and widely applicable information criterion in singular learning theory. 
-#' The Journal of Machine Learning Research, 11, 3571-3594.
-#' 
 #' @export
 WAIC <- function(x, ...) {
   UseMethod("WAIC")
@@ -129,7 +13,7 @@ WAIC <- function(x, ...) {
 #' Perform exact K-fold cross-validation by refitting the model \eqn{K}
 #' times each leaving out one-\eqn{K}th of the original data.
 #' 
-#' @inheritParams LOO
+#' @inheritParams loo.brmsfit
 #' @param K The number of subsets of equal (if possible) size
 #'   into which the data will be randomly partitioned for performing
 #'   \eqn{K}-fold cross-validation. The model is refit \code{K} times, each time
@@ -221,13 +105,14 @@ compute_ics <- function(models, ic = c("loo", "waic", "psis", "psislw", "kfold")
         out[[i]] <- do.call(compute_ic, args) 
       }
     }
+    compare <- as_one_logical(compare)
     if (compare) {
       out <- compare_ic(x = out)
     }
     class(out) <- "iclist"
   } else {
     ic_obj <- models[[1]][[ic]]
-    stopifnot(length(use_stored_ic) == 1L)
+    use_stored_ic <- as_one_logical(use_stored_ic)
     if (use_stored_ic && is.ic(ic_obj)) {
       out <- ic_obj
       out$model_name <- names(models)
@@ -255,7 +140,7 @@ compute_ic <- function(x, ic = c("loo", "waic", "psis", "kfold"),
   #   an object of class 'ic' which inherits from class 'loo'
   ic <- match.arg(ic)
   if (ic == "kfold") {
-    IC <- do.call(kfold_internal, list(x, ...))
+    out <- do.call(kfold_internal, list(x, ...))
   } else {
     contains_samples(x)
     pointwise <- as_one_logical(pointwise)
@@ -272,29 +157,30 @@ compute_ic <- function(x, ic = c("loo", "waic", "psis", "kfold"),
       loo_args$log_ratios <- -loo_args$x
       loo_args$x <- NULL
     }
-    IC <- SW(do.call(eval2(paste0("loo::", ic)), loo_args))
+    out <- SW(do.call(eval2(paste0("loo::", ic)), loo_args))
   }
-  IC$model_name <- model_name
-  class(IC) <- c("ic", class(IC))
+  out$model_name <- model_name
+  class(out) <- c("ic", class(out))
+  attr(out, "yhash") <- hash_response(x)
   if (ic == "loo") {
     if (reloo) {
-      reloo_args <- nlist(x = IC, fit = x, k_threshold, check = FALSE)
-      IC <- do.call(reloo.loo, c(reloo_args, ...))
+      reloo_args <- nlist(x = out, fit = x, k_threshold, check = FALSE)
+      out <- do.call(reloo.loo, c(reloo_args, ...))
     } else {
-      n_bad_obs <- length(loo::pareto_k_ids(IC, threshold = k_threshold))
+      n_bad_obs <- length(loo::pareto_k_ids(out, threshold = k_threshold))
       recommend_loo_options(n_bad_obs, k_threshold, model_name) 
     }
   }
-  IC
+  out
 }
 
 #' Compare Information Criteria of Different Models
 #'
 #' Compare information criteria of different models fitted
-#' with \code{\link{WAIC}} or \code{\link{LOO}}.
+#' with \code{\link{WAIC}} or \code{\link{loo}}.
 #' 
 #' @param ... At least two objects returned by 
-#'   \code{\link{WAIC}} or \code{\link{LOO}}.
+#'   \code{\link{WAIC}} or \code{\link{loo}}.
 #'   Alternatively, \code{brmsfit} objects with information 
 #'   criteria precomputed via \code{\link{add_ic}}
 #'   may be passed, as well.
@@ -310,7 +196,7 @@ compute_ic <- function(x, ic = c("loo", "waic", "psis", "kfold"),
 #' 
 #' @seealso 
 #'   \code{\link{WAIC}}, 
-#'   \code{\link{LOO}},
+#'   \code{\link{loo}},
 #'   \code{\link{add_ic}},
 #'   \code{\link[loo:compare]{compare}}
 #'   
@@ -351,7 +237,7 @@ compare_ic <- function(..., x = NULL, ic = c("loo", "waic", "kfold")) {
   if (length(x) < 2L) {
     stop2("Expecting at least two objects.")
   }
-  ics <- unname(sapply(x, function(y) names(y)[3]))
+  ics <- unname(sapply(x, function(y) rownames(y$estimates)[3]))
   if (!all(ics %in% ics[1])) {
     stop2("All inputs should be from the same criterion.")
   }
@@ -366,6 +252,14 @@ compare_ic <- function(..., x = NULL, ic = c("loo", "waic", "kfold")) {
       stop2("The number of subsets differs across kfold objects.")
     }
   }
+  yhash <- lapply(x, attr, which = "yhash")
+  yhash_check <- ulapply(yhash, is_equal, yhash[[1]])
+  if (!all(yhash_check)) {
+    warning2(
+      "Model comparisons are likely invalid as the response ", 
+      "values of at least two models do not match."
+    )
+  }
   names(x) <- ulapply(x, "[[", "model_name")
   n_models <- length(x)
   ic_diffs <- matrix(0, nrow = n_models * (n_models - 1) / 2, ncol = 2)
@@ -374,8 +268,8 @@ compare_ic <- function(..., x = NULL, ic = c("loo", "waic", "kfold")) {
   n <- 1
   for (i in seq_len(n_models - 1)) {
     for (j in (i + 1):n_models) {
-      temp <- loo::compare(x[[j]], x[[i]])
-      ic_diffs[n, ] <- c(-2 * temp[["elpd_diff"]], 2 * temp[["se"]]) 
+      tmp <- loo::compare(x[[j]], x[[i]])
+      ic_diffs[n, ] <- c(-2 * tmp[["elpd_diff"]], 2 * tmp[["se"]]) 
       rnames[n] <- paste(names(x)[i], "-", names(x)[j])
       n <- n + 1
     }
@@ -471,38 +365,6 @@ add_waic <- function(x, ...) {
   add_ic(x, ic = "waic", ...)
 }
 
-set_pointwise <- function(x, pointwise = NULL, newdata = NULL, 
-                          subset = NULL, thres = 1e+08) {
-  # set the pointwise argument based on the model size
-  # Args:
-  #   x: a brmsfit object
-  #   newdata: optional data.frame containing new data
-  #   subset: a vector to indicate a subset of the posterior samples
-  #   thres: threshold above which pointwise is set to TRUE
-  # Returns:
-  #   TRUE or FALSE
-  if (!is.null(pointwise)) {
-    pointwise <- as_one_logical(pointwise)
-  } else {
-    nsamples <- nsamples(x, subset = subset)
-    if (is.data.frame(newdata)) {
-      nobs <- nrow(newdata)
-    } else {
-      nobs <- nobs(x)
-    }
-    pointwise <- nsamples * nobs > thres
-    if (pointwise) {
-      message(
-        "Switching to pointwise evaluation to reduce ",  
-        "RAM requirements. This will likely increase ",
-        "computation time. You may overwrite the default ",
-        "behavior with the 'pointwise' argument."
-      )
-    }
-  }
-  pointwise
-}
-
 args_not_for_reloo <- function() {
   # arguments not usable with 'reloo'
   # the same arguments cannot be used in add_ic
@@ -510,32 +372,34 @@ args_not_for_reloo <- function() {
     "allow_new_levels", "sample_new_levels", "new_objects")
 }
 
-match_response <- function(models) {
+hash_response <- function(x, ...) {
+  # create a hash based on the response of a model
+  require_package("digest")
+  stopifnot(is.brmsfit(x))
+  sdata <- standata(x, internal = TRUE)
+  add_funs <- lsp("brms", what = "exports", pattern = "^resp_")
+  regex <- c("Y", sub("^resp_", "", add_funs))
+  regex <- paste0("(", regex, ")", collapse = "|")
+  regex <- paste0("^(", regex, ")(_|$)")
+  out <- sdata[grepl(regex, names(sdata))]
+  out <- as.matrix(as.data.frame(rmNULL(out)))
+  out <- p(out, attr(sdata, "old_order"))
+  digest::sha1(x = out, ...)
+}
+
+match_response <- function(models, ...) {
   # compare the response parts of multiple brmsfit objects
   # Args:
   #   models: A list of brmsfit objects
+  #   ...: passed to hash_response
   # Returns:
   #   TRUE if the response parts of all models match and FALSE else
   if (length(models) <= 1L) {
     out <- TRUE  
   } else {
-    add_funs <- lsp("brms", what = "exports", pattern = "^resp_")
-    match_vars <- c("Y", sub("^resp_", "", add_funs))
-    .match_fun <- function(x, y) {
-      # checks if all relevant parts of the response are the same 
-      # Args:
-      #   x, y: named lists as returned by standata
-      old_order_x <- attr(x, "old_order")
-      old_order_y <- attr(y, "old_order")
-      all(ulapply(match_vars, function(v) {
-        a <- p(as.vector(x[[v]]), old_order_x)
-        b <- p(as.vector(y[[v]]), old_order_y)
-        is_equal(a, b)
-      }))
-    }
-    sdatas <- lapply(models, standata, internal = TRUE)
-    matches <- ulapply(sdatas[-1], .match_fun, y = sdatas[[1]]) 
-    if (all(matches)) {
+    yhash <- lapply(models, hash_response, ...)
+    yhash_check <- ulapply(yhash, is_equal, yhash[[1]])
+    if (all(yhash_check)) {
       out <- TRUE
     } else {
       out <- FALSE
@@ -545,7 +409,7 @@ match_response <- function(models) {
 }
 
 validate_models <- function(models, model_names, sub_names) {
-  # validate models passed to LOO and related methods
+  # validate models passed to loo and related methods
   # Args:
   #   models: list of fitted model objects
   #   model_names: names specified by the user
@@ -564,12 +428,6 @@ validate_models <- function(models, model_names, sub_names) {
       stop2("Object '", names(models)[i], "' is not of class 'brmsfit'.")
     }
   }
-  if (!match_response(models)) {
-    warning2(
-      "Model comparisons are likely invalid as the response ", 
-      "parts of at least two models do not match."
-    )
-  }
   models
 }
 
@@ -579,7 +437,7 @@ validate_models <- function(models, model_names, sub_names) {
 #' for which approximate leave-one-out cross-validation may
 #' return incorrect results.
 #' 
-#' @inheritParams LOO
+#' @inheritParams loo.brmsfit
 #' @param x An \R object typically of class \code{loo}.
 #' @param fit An \R object typically of class \code{brmsfit}.
 #' @param k_threshold The threshold at which pareto \eqn{k} 
