@@ -129,82 +129,46 @@ predict_skew_normal <- function(i, draws, ...) {
 predict_gaussian_mv <- function(i, draws, ...) {
   Mu <- get_Mu(draws, i = i)
   Sigma <- get_Sigma(draws, i = i)
-  rmn <- function(s) {
+  .predict <- function(s) {
     rmulti_normal(1, mu = Mu[s, ], Sigma = Sigma[s, , ])
   }
-  do.call(rbind, lapply(1:draws$nsamples, rmn))
+  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_student_mv <- function(i, draws, ...) {
   nu <- get_dpar(draws, "nu", i = i)
   Mu <- get_Mu(draws, i = i)
   Sigma <- get_Sigma(draws, i = i)
-  rmst <- function(s) {
+  .predict <- function(s) {
     rmulti_student_t(1, df = nu[s], mu = Mu[s, ], Sigma = Sigma[s, , ])
   }
-  do.call(rbind, lapply(1:draws$nsamples, rmst))
+  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_gaussian_cov <- function(i, draws, ...) {
-  # currently, only ARMA1 processes are implemented
   obs <- with(draws$ac, begin_tg[i]:(begin_tg[i] + nobs_tg[i] - 1))
   mu <- as.matrix(get_dpar(draws, "mu", i = obs))
-  args <- list(
-    sigma = get_dpar(draws, "sigma", i = obs),
-    se = draws$data$se[obs], nrows = length(obs)
-  )
-  ar <- draws$ac$ar
-  ma <- draws$ac$ma
-  if (!is.null(ar) && is.null(ma)) {
-    args$ar <- ar
-    Sigma <- do.call(get_cov_matrix_ar1, args)
-  } else if (is.null(ar) && !is.null(ma)) {
-    args$ma <- ma
-    Sigma <- do.call(get_cov_matrix_ma1, args)
-  } else if (!is.null(ar) && !is.null(ma)) {
-    args[c("ar", "ma")] <- draws$ac[c("ar", "ma")]
-    Sigma <- do.call(get_cov_matrix_arma1, args)
-  } else {
-    Sigma <- do.call(get_cov_matrix_ident, args)
-  }
-  .fun <- function(s) {
+  Sigma <- get_cov_matrix_arma(draws, obs)
+  .predict <- function(s) {
     rmulti_normal(1, mu = mu[s, ], Sigma = Sigma[s, , ])
   }
-  do.call(rbind, lapply(1:draws$nsamples, .fun))
+  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_student_cov <- function(i, draws, ...) {
-  # currently, only ARMA1 processes are implemented
   obs <- with(draws$ac, begin_tg[i]:(begin_tg[i] + nobs_tg[i] - 1))
-  args <- list(
-    sigma = get_dpar(draws, "sigma", i = obs), 
-    se = draws$data$se[obs], nrows = length(obs)
-  )
-  ar <- draws$ac$ar
-  ma <- draws$ac$ma
-  if (!is.null(ar) && is.null(ma)) {
-    args$ar <- ar
-    Sigma <- do.call(get_cov_matrix_ar1, args)
-  } else if (is.null(ar) && !is.null(ma)) {
-    args$ma <- ma
-    Sigma <- do.call(get_cov_matrix_ma1, args)
-  } else if (!is.null(ar) && !is.null(ma)) {
-    args[c("ar", "ma")] <- draws$ac[c("ar", "ma")]
-    Sigma <- do.call(get_cov_matrix_arma1, args)
-  } else {
-    Sigma <- do.call(get_cov_matrix_ident, args)
-  }
-  mu <- as.matrix(get_dpar(draws, "mu", i = obs))
   nu <- as.matrix(get_dpar(draws, "nu", i = obs))
-  .fun <- function(s) {
+  mu <- as.matrix(get_dpar(draws, "mu", i = obs))
+  Sigma <- get_cov_matrix_arma(draws, obs)
+  .predict <- function(s) {
     rmulti_student_t(1, df = nu[s, ], mu = mu[s, ], Sigma = Sigma[s, , ])
   }
-  do.call(rbind, lapply(1:draws$nsamples, .fun))
+  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_gaussian_lagsar <- function(i, draws, ...) {
   stopifnot(i == 1)
-  .predict_gaussian_lagsar <- function(s) {
+  .predict <- function(s) {
     W_new <- with(draws, diag(nobs) - ac$lagsar[s] * ac$W)
     mu <- as.numeric(solve(W_new) %*% mu[s, ])
     Sigma <- solve(crossprod(W_new)) * sigma[s]^2
@@ -212,12 +176,12 @@ predict_gaussian_lagsar <- function(i, draws, ...) {
   }
   mu <- get_dpar(draws, "mu")
   sigma <- get_dpar(draws, "sigma")
-  do.call(rbind, lapply(1:draws$nsamples, .predict_gaussian_lagsar))
+  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_student_lagsar <- function(i, draws, ...) {
   stopifnot(i == 1)
-  .predict_student_lagsar <- function(s) {
+  .predict <- function(s) {
     W_new <- with(draws, diag(nobs) - ac$lagsar[s] * ac$W)
     mu <- as.numeric(solve(W_new) %*% mu[s, ])
     Sigma <- solve(crossprod(W_new)) * sigma[s]^2
@@ -226,24 +190,24 @@ predict_student_lagsar <- function(i, draws, ...) {
   mu <- get_dpar(draws, "mu")
   sigma <- get_dpar(draws, "sigma")
   nu <- get_dpar(draws, "nu")
-  do.call(rbind, lapply(1:draws$nsamples, .predict_student_lagsar))
+  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_gaussian_errorsar <- function(i, draws, ...) {
   stopifnot(i == 1)
-  .predict_gaussian_errorsar <- function(s) {
+  .predict <- function(s) {
     W_new <- with(draws, diag(nobs) - ac$errorsar[s] * ac$W)
     Sigma <- solve(crossprod(W_new)) * sigma[s]^2
     rmulti_normal(1, mu = mu[s, ], Sigma = Sigma)
   }
   mu <- get_dpar(draws, "mu")
   sigma <- get_dpar(draws, "sigma")
-  do.call(rbind, lapply(1:draws$nsamples, .predict_gaussian_errorsar))
+  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_student_errorsar <- function(i, draws, ...) {
   stopifnot(i == 1)
-  .predict_student_errorsar <- function(s) {
+  .predict <- function(s) {
     W_new <- with(draws, diag(nobs) - ac$errorsar[s] * ac$W)
     Sigma <- solve(crossprod(W_new)) * sigma[s]^2
     rmulti_student_t(1, df = nu[s], mu = mu[s, ], Sigma = Sigma)
@@ -251,26 +215,26 @@ predict_student_errorsar <- function(i, draws, ...) {
   mu <- get_dpar(draws, "mu")
   sigma <- get_dpar(draws, "sigma")
   nu <- get_dpar(draws, "nu")
-  do.call(rbind, lapply(1:draws$nsamples, .predict_student_errorsar))
+  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_gaussian_fixed <- function(i, draws, ...) {
   stopifnot(i == 1)
   mu <- as.matrix(get_dpar(draws, "mu"))
-  .fun <- function(s) {
+  .predict <- function(s) {
     rmulti_normal(1, mu = mu[s, ], Sigma = draws$ac$V)
   }
-  do.call(rbind, lapply(1:draws$nsamples, .fun))
+  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_student_fixed <- function(i, draws, ...) {
   stopifnot(i == 1)
   mu <- as.matrix(get_dpar(draws, "mu"))
   nu <- as.matrix(get_dpar(draws, "nu"))
-  .fun <- function(s) {
+  .predict <- function(s) {
     rmulti_student_t(1, df = nu[s, ], mu = mu[s, ], Sigma = draws$ac$V)
   }
-  do.call(rbind, lapply(1:draws$nsamples, .fun))
+  do.call(rbind, lapply(seq_len(draws$nsamples), .predict))
 }
 
 predict_binomial <- function(i, draws, ntrys = 5, ...) {
